@@ -6,15 +6,17 @@ import os
 import shutil
 import tempfile
 
-from hotsos.core.log import log
-from hotsos.core.issues import IssuesManager
-from hotsos.core.host_helpers.cli import CLIHelper
 from hotsos.core.config import HotSOSConfig
+from hotsos.core.host_helpers.cli import CLIHelper
+from hotsos.core.issues import IssuesManager
+from hotsos.core.log import log
 from hotsos.core import plugintools
-
-from hotsos.plugin_extensions.lxd.summary import LXDSummary
-from hotsos.plugin_extensions.mysql.summary import MySQLSummary
 from hotsos.plugin_extensions.juju.summary import JujuSummary
+from hotsos.plugin_extensions.kernel import summary as kern_summary
+from hotsos.plugin_extensions.kubernetes.summary import KubernetesSummary
+from hotsos.plugin_extensions.lxd.summary import LXDSummary
+from hotsos.plugin_extensions.maas.summary import MAASSummary
+from hotsos.plugin_extensions.mysql.summary import MySQLSummary
 from hotsos.plugin_extensions.openstack import (
     summary as ost_summary,
     service_features,
@@ -26,16 +28,11 @@ from hotsos.plugin_extensions.openstack.agent import (
     events as agent_events,
     exceptions as agent_exceptions,
 )
-
 from hotsos.plugin_extensions.openvswitch import (
     summary as ovs_summary,
     event_checks,
 )
-from hotsos.plugin_extensions.system.summary import SystemSummary
-from hotsos.plugin_extensions.system.checks import SYSCtlChecks
-from hotsos.plugin_extensions.maas.summary import MAASSummary
-from hotsos.plugin_extensions.kernel import summary as kern_summary
-from hotsos.plugin_extensions.kubernetes.summary import KubernetesSummary
+from hotsos.plugin_extensions.pacemaker.summary import PacemakerSummary
 from hotsos.plugin_extensions.rabbitmq.summary import RabbitMQSummary
 from hotsos.plugin_extensions.sosreport.summary import SOSReportSummary
 from hotsos.plugin_extensions.storage import (
@@ -43,8 +40,9 @@ from hotsos.plugin_extensions.storage import (
     ceph_event_checks,
     bcache_summary,
 )
+from hotsos.plugin_extensions.system.checks import SYSCtlChecks
+from hotsos.plugin_extensions.system.summary import SystemSummary
 from hotsos.plugin_extensions.vault.summary import VaultSummary
-from hotsos.plugin_extensions.pacemaker.summary import PacemakerSummary
 
 
 class HotSOSSummary(plugintools.PluginPartBase):
@@ -102,8 +100,7 @@ PLUGIN_CATALOG = {'hotsos': {
                   'mysql': {
                      'summary': {
                          'objects': [MySQLSummary],
-                         'part_yaml_offset': 0},
-                      },
+                         'part_yaml_offset': 0}},
                   'openstack': {
                      'summary': {
                          'objects': [ost_summary.OpenstackSummary],
@@ -249,13 +246,13 @@ class OutputManager(object):
 
         if mode == 'short':
             return self._get_short_format(summary)
-        elif mode == 'very-short':
+        if mode == 'very-short':
             return self._get_very_short_format(summary)
 
         log.warning("Unknown minimalmode '%s'", mode)
         return summary
 
-    def get(self, format='yaml', html_escape=False, minimal_mode=None,
+    def get(self, fmt='yaml', html_escape=False, minimal_mode=None,
             plugin=None, max_level=2):
         if plugin:
             filtered = {plugin: self._summary[plugin]}
@@ -265,18 +262,18 @@ class OutputManager(object):
         if minimal_mode:
             filtered = self.minimise(filtered, minimal_mode)
 
-        if format not in self.SUMMARY_FORMATS:
-            raise Exception("unsupported summary format '{}'".format(format))
+        if fmt not in self.SUMMARY_FORMATS:
+            raise Exception("unsupported summary format '{}'".format(fmt))
 
         hostname = CLIHelper().hostname() or ""
-        log.debug('Saving summary as %s', format)
-        if format == 'yaml':
+        log.debug('Saving summary as %s', fmt)
+        if fmt == 'yaml':
             filtered = plugintools.yaml_dump(filtered)
-        elif format == 'json':
+        elif fmt == 'json':
             filtered = json.dumps(filtered, indent=2, sort_keys=True)
-        elif format == 'markdown':
+        elif fmt == 'markdown':
             filtered = plugintools.MarkdownFormatter().dump(filtered)
-        elif format == 'html':
+        elif fmt == 'html':
             filtered = plugintools.HTMLFormatter(
                                             hostname=hostname,
                                             max_level=max_level).dump(filtered)
@@ -287,9 +284,9 @@ class OutputManager(object):
 
         return filtered
 
-    def _save(self, path, format, html_escape=None, minimal_mode=None,
+    def _save(self, path, fmt, html_escape=None, minimal_mode=None,
               plugin=None):
-        content = self.get(format=format, html_escape=html_escape,
+        content = self.get(fmt=fmt, html_escape=html_escape,
                            minimal_mode=minimal_mode, plugin=plugin)
         with open(path, 'w', encoding='utf-8') as fd:
             fd.write(content)
@@ -310,9 +307,9 @@ class OutputManager(object):
 
         for minimal_mode in ['full', 'short', 'very-short']:
             _minimal_mode = minimal_mode.replace('-', '_')
-            for format in self.SUMMARY_FORMATS:
+            for fmt in self.SUMMARY_FORMATS:
                 output_path = os.path.join(output_root, name, 'summary',
-                                           _minimal_mode, format)
+                                           _minimal_mode, fmt)
                 if minimal_mode == 'full':
                     minimal_mode = None
 
@@ -322,18 +319,18 @@ class OutputManager(object):
                 for plugin in self._summary:
                     path = os.path.join(output_path,
                                         "hotsos-summary.{}.{}".format(plugin,
-                                                                      format))
-                    self._save(path, format, html_escape=html_escape,
+                                                                      fmt))
+                    self._save(path, fmt, html_escape=html_escape,
                                minimal_mode=minimal_mode, plugin=plugin)
 
                 path = os.path.join(output_path,
-                                    "hotsos-summary.all.{}".format(format))
-                self._save(path, format, html_escape=html_escape,
+                                    "hotsos-summary.all.{}".format(fmt))
+                self._save(path, fmt, html_escape=html_escape,
                            minimal_mode=minimal_mode)
 
                 if not minimal_mode:
                     dst = os.path.join(output_root, '{}.summary.{}'.
-                                       format(name, format))
+                                       format(name, fmt))
                     if os.path.exists(dst):
                         os.remove(dst)
 
@@ -384,12 +381,12 @@ class HotSOSClient(object):
                                                        dir=global_tmp)
 
     def _run(self, plugin):
-        log.debug("running plugin %s", plugin)
         if plugin not in PLUGIN_CATALOG:
             raise Exception("unknown plugin {}".format(plugin))
 
+        log.name = 'hotsos.plugin.{}'.format(plugin)
+        log.debug("running plugin %s", plugin)
         HotSOSConfig.plugin_name = plugin
-        log.name = 'plugin.{}'.format(plugin)
         parts = PLUGIN_CATALOG[plugin]
         return plugintools.PluginRunner(parts).run()
 
