@@ -1,4 +1,7 @@
-from .cli import CLIHelper  # noqa: F403,F401
+from .cli import (  # noqa: F403,F401
+    CLIHelper,
+    CLIHelperFile,
+)
 from .config import (  # noqa: F403,F401
     ConfigBase,
     SectionalConfigBase,
@@ -13,13 +16,15 @@ from .packaging import (  # noqa: F403,F401
     DockerImageHelper,
     SnapPackageHelper,
 )
+from .pebble import (  # noqa: F403,F401
+    PebbleHelper,
+)
 from .ssl import (  # noqa: F403,F401
     SSLCertificate,
     SSLCertificatesHelper,
 )
 from .systemd import (  # noqa: F403,F401
     SystemdHelper,
-    SVC_EXPR_TEMPLATES,
 )
 from .uptime import (  # noqa: F403,F401
     UptimeHelper,
