@@ -1,15 +1,22 @@
+import abc
 import datetime
 import glob
 import json
 import os
+import pathlib
 import pickle
 import re
 import subprocess
 import tempfile
+from functools import cached_property
 
-from hotsos.core.log import log
 from hotsos.core.config import HotSOSConfig
 from hotsos.core.host_helpers.common import HostHelpersBase
+from hotsos.core.log import log
+
+CLI_COMMON_EXCEPTIONS = (OSError, subprocess.CalledProcessError,
+                         subprocess.TimeoutExpired,
+                         json.JSONDecodeError)
 
 
 class CLIExecError(Exception):
@@ -28,7 +35,12 @@ def catch_exceptions(*exc_types):
             try:
                 return f(*args, **kwargs)
             except exc_types as exc:
-                log.debug("%s: %s", type(exc), exc)
+                msg = "{}: {}".format(type(exc), exc)
+                if isinstance(exc, subprocess.TimeoutExpired):
+                    log.info(msg)
+                else:
+                    log.debug(msg)
+
                 if type(exc) == json.JSONDecodeError:
                     raise CLIExecError(return_value={}) from exc
 
@@ -44,8 +56,8 @@ class SourceNotFound(Exception):
 
 
 class CommandNotFound(Exception):
-    def __init__(self, cmd):
-        self.msg = "command not found in catalog: '{}'".format(cmd)
+    def __init__(self, cmd, msg):
+        self.msg = "command '{}' not found in catalog: '{}'".format(cmd, msg)
 
     def __str__(self):
         return self.msg
@@ -53,7 +65,7 @@ class CommandNotFound(Exception):
 
 class NullSource(object):
     def __call__(self, *args, **kwargs):
-        return []
+        return CmdOutput([])
 
 
 def run_pre_exec_hooks(f):
@@ -101,7 +113,14 @@ def reset_command(f):
     return reset_command_inner
 
 
+class CmdOutput(object):
+    def __init__(self, value, source=None):
+        self.value = value
+        self.source = source
+
+
 class CmdBase(object):
+    """ Base class for all command source types. """
 
     def __init__(self):
         self.hooks = {}
@@ -116,10 +135,10 @@ class CmdBase(object):
         """
         raise NotImplementedError
 
-    """ Base class for all command source types. """
     @classmethod
     def safe_readlines(cls, path):
-        return open(path, 'r', errors="surrogateescape").readlines()
+        with open(path, 'r', errors="surrogateescape") as fd:
+            return fd.readlines()
 
     def register_hook(self, name, f):
         """
@@ -153,12 +172,11 @@ class BinCmd(CmdBase):
         self.json_decode = self.original_json_decode
         self.singleline = self.original_singleline
 
-    @catch_exceptions(OSError, subprocess.CalledProcessError,
-                      json.JSONDecodeError)
+    @catch_exceptions(*CLI_COMMON_EXCEPTIONS)
     @reset_command
     @run_post_exec_hooks
     @run_pre_exec_hooks
-    def __call__(self, *args, **kwargs):
+    def __call__(self, *args, skip_json_decode=False, **kwargs):
         cmd = self.cmd
         if args:
             cmd = cmd.format(*args)
@@ -167,40 +185,53 @@ class BinCmd(CmdBase):
             cmd = cmd.format(**kwargs)
 
         _cmd = cmd.split() + self.original_cmd_extras
-        output = subprocess.check_output(_cmd, stderr=subprocess.STDOUT)
+        out = subprocess.run(_cmd, timeout=HotSOSConfig.command_timeout,
+                             capture_output=True, check=False)
+        output = out.stdout
+        if out.stderr:
+            output += out.stderr
 
-        if self.json_decode:
-            return json.loads(output.decode('UTF-8'))
+        if out.returncode != 0:
+            log.info("command '%s' exited with non-zero code '%s'", cmd,
+                     out.returncode)
+
+        try:
+            output = output.decode('UTF-8')
+        except UnicodeDecodeError:
+            log.exception("failed to decode command output for '%s'", cmd)
+            output = ''
+
+        if self.json_decode and not skip_json_decode:
+            return CmdOutput(json.loads(output))
 
         if self.singleline:
-            return output.decode('UTF-8').strip()
+            return CmdOutput(output.strip())
 
-        return output.decode('UTF-8').splitlines(keepends=True)
+        return CmdOutput(output.splitlines(keepends=True))
 
 
 class FileCmd(CmdBase):
     TYPE = "FILE"
 
-    def __init__(self, path, safe_decode=False, json_decode=False,
-                 singleline=False):
+    def __init__(self, path, json_decode=False,
+                 singleline=False, decode_error_handling=None):
         self.original_path = os.path.join(HotSOSConfig.data_root, path)
-        self.original_safe_decode = safe_decode
         self.original_json_decode = json_decode
         self.original_singleline = singleline
+        self.original_decode_error_handling = decode_error_handling
         super().__init__()
 
     def reset(self):
         self.path = self.original_path
-        self.safe_decode = self.original_safe_decode
         self.json_decode = self.original_json_decode
         self.singleline = self.original_singleline
+        self.decode_error_handling = self.original_decode_error_handling
 
-    @catch_exceptions(OSError, subprocess.CalledProcessError,
-                      json.JSONDecodeError)
+    @catch_exceptions(*CLI_COMMON_EXCEPTIONS)
     @reset_command
     @run_post_exec_hooks
     @run_pre_exec_hooks
-    def __call__(self, *args, **kwargs):
+    def __call__(self, *args, skip_load_contents=False, **kwargs):
         if args:
             self.path = self.path.format(*args)
 
@@ -210,28 +241,51 @@ class FileCmd(CmdBase):
         if not os.path.exists(self.path):
             raise SourceNotFound()
 
-        # NOTE: any post-exec hooks much be aware that their input will be
-        # defined by the following.
-        if self.safe_decode:
-            # Some content can result in UnicodeDecodeError so use
-            # surrogateescape but only seldom.
-            output = self.safe_readlines(self.path)
-        elif self.json_decode:
-            output = json.load(open(self.path))
-        else:
-            output = open(self.path, 'r').readlines()
-            if self.singleline:
-                return output[0].strip()
+        if skip_load_contents:
+            CmdOutput(None, self.path)
 
-        return output
+        # NOTE: any post-exec hooks must be aware that their input will be
+        # defined by the following.
+        if self.json_decode:
+            with open(self.path) as fd:
+                output = json.load(fd)
+        else:
+            output = []
+            ln = 0
+            with open(self.path, 'rb') as fd:
+                for line in fd:
+                    ln += 1
+                    try:
+                        decode_kwargs = {}
+                        if self.decode_error_handling:
+                            decode_kwargs['errors'] = \
+                                                     self.decode_error_handling
+
+                        _out = line.decode(**decode_kwargs)
+                        output.append(_out)
+                    except UnicodeDecodeError:
+                        # maintain line count but store empty line.
+                        # we could in the future consider other decode options
+                        # as a fallback.
+                        output.append('')
+                        log.exception("failed to decode line %s "
+                                      "(decode_error_handling=%s)", ln,
+                                      self.decode_error_handling)
+
+                    if self.singleline:
+                        break
+
+            if self.singleline:
+                return CmdOutput(output[0].strip(), self.path)
+
+        return CmdOutput(output, self.path)
 
 
 class BinFileCmd(FileCmd):
     """ This is used when we are executing an actual binary/command against a
     file. """
 
-    @catch_exceptions(OSError, subprocess.CalledProcessError,
-                      json.JSONDecodeError)
+    @catch_exceptions(*CLI_COMMON_EXCEPTIONS)
     @reset_command
     @run_post_exec_hooks
     @run_pre_exec_hooks
@@ -252,15 +306,17 @@ class BinFileCmd(FileCmd):
         env = {}
         try:
             env['TZ'] = DateFileCmd('sos_commands/date/date',
-                                    singleline=True)(format="+%Z")
+                                    singleline=True)(format="+%Z").value
         except SourceNotFound:
             pass
 
         # Now split into a command and run
         output = subprocess.check_output(self.path.split(),
+                                         timeout=HotSOSConfig.command_timeout,
                                          stderr=subprocess.STDOUT, env=env)
 
-        return output.decode('UTF-8').splitlines(keepends=True)
+        output = output.decode('UTF-8')
+        return CmdOutput(output.splitlines(keepends=True))
 
 
 class JournalctlBase(object):
@@ -359,7 +415,7 @@ class OVSOFCTLBinCmd(BinCmd):
             except CLIExecError:
                 log.debug("ofctl command with protocol version %s failed", ver)
 
-        return []
+        return CmdOutput([])
 
 
 class DateBinCmd(BinCmd):
@@ -371,14 +427,14 @@ class DateBinCmd(BinCmd):
     def format_date_cmd(self, **kwargs):
         """ Add formatting to date command. """
         no_format = kwargs.get('no_format', False)
-        format = kwargs.get('format')
-        if not no_format and format is None:
-            format = '+%s'
+        fmt = kwargs.get('format')
+        if not no_format and fmt is None:
+            fmt = '+%s'
 
         self.cmd = '{} --utc'.format(self.cmd)
-        if format:
+        if fmt:
             # this can't get split() so add to the end of the command list
-            self.original_cmd_extras = [format]
+            self.original_cmd_extras = [fmt]
 
 
 class DateFileCmd(FileCmd):
@@ -388,20 +444,24 @@ class DateFileCmd(FileCmd):
         self.register_hook("post-exec", self.format_date)
 
     def format_date(self, output, **kwargs):
-        """ Apply some post-processing to the date output. """
+        """ Apply some post-processing to the date output.
+
+        @param output: CmdOutput object
+        """
         no_format = kwargs.get('no_format', False)
-        format = kwargs.get('format')
-        if not no_format and format is None:
-            format = '+%s'
+        fmt = kwargs.get('format')
+        if not no_format and fmt is None:
+            fmt = '+%s'
 
         ret = re.match(r"^(\S+ \S*\s*[0-9]+ [0-9:]+)\s*"
                        r"([A-Z]*|[+-]?[0-9]*)?"
                        r"\s*([0-9]+)$",
-                       output)
+                       output.value)
 
         if ret is None:
-            log.error("%s has invalid date string '%s'", self.path, output)
-            return ""
+            log.error("%s has invalid date string '%s'", self.path,
+                      output.value)
+            return CmdOutput('', self.path)
 
         tz = ret[2]
         # NOTE: date command doesn't recognise HKT for some reason so we
@@ -412,16 +472,17 @@ class DateFileCmd(FileCmd):
         # Include tz if available and then convert to utc
         date = "{} {} {}".format(ret[1], tz, ret[3])
         cmd = ["date", "--utc", "--date={}".format(date)]
-        if format:
-            cmd.append(format)
+        if fmt:
+            cmd.append(fmt)
 
-        output = subprocess.check_output(cmd)
+        output = subprocess.check_output(cmd,
+                                         timeout=HotSOSConfig.command_timeout)
         # date sometimes adds multiple whitespaces between fields so collapse
         # them.
         output = re.compile(r"\s+").sub(' ', output.decode('UTF-8'))
         ret = output.splitlines(keepends=True)[0]
         # always singleline so always strip trailing newline
-        return ret.strip()
+        return CmdOutput(ret.strip(), self.path)
 
 
 class CephJSONFileCmd(FileCmd):
@@ -461,6 +522,9 @@ class CephJSONFileCmd(FileCmd):
                 self.path = tmp.name
 
     def cleanup(self, output, **kwargs):  # pylint: disable=W0613
+        """
+        @param output: CmdOutput object
+        """
         if self.orig_path:
             os.unlink(self.path)
             self.path = self.orig_path
@@ -471,7 +535,7 @@ class CephJSONFileCmd(FileCmd):
 
 class SourceRunner(object):
 
-    def __init__(self, cmdkey, sources, cache):
+    def __init__(self, cmdkey, sources, cache, output_file=None):
         """
         @param cmdkey: unique key identifying this command.
         @param sources: list of command source implementations.
@@ -480,6 +544,72 @@ class SourceRunner(object):
         self.cmdkey = cmdkey
         self.sources = sources
         self.cache = cache
+        self.output_file = output_file
+        # Command output can differ between CLIHelper and CLIHelperFile so we
+        # need to cache them separately.
+        if output_file:
+            self.cache_cmdkey = "{}.file".format(cmdkey)
+        else:
+            self.cache_cmdkey = cmdkey
+
+    def bsource(self, *args, **kwargs):
+        # binary sources only apply if data_root is system root
+        bin_out = None
+        for bsource in [s for s in self.sources if s.TYPE == "BIN"]:
+            cache = False
+            # NOTE: we currently only support caching commands with no
+            #       args.
+            if not any([args, kwargs]):
+                cache = True
+                out = self.cache.load(self.cache_cmdkey)
+                if out is not None:
+                    return out
+
+            try:
+                if self.output_file:
+                    # don't decode if we are going to be saving to a file
+                    kwargs['skip_json_decode'] = True
+
+                bin_out = bsource(*args, **kwargs)
+                if cache and bin_out is not None:
+                    try:
+                        self.cache.save(self.cache_cmdkey, bin_out)
+                    except pickle.PicklingError as exc:
+                        log.info("unable to cache command '%s' output: %s",
+                                 self.cmdkey, exc)
+
+                # if command executed but returned nothing that still counts
+                # as success.
+                break
+            except CLIExecError as exc:
+                bin_out = CmdOutput(exc.return_value)
+
+        return bin_out
+
+    def fsource(self, *args, **kwargs):
+        for fsource in [s for s in self.sources if s.TYPE == "FILE"]:
+            try:
+                skip_load_contents = False
+                if self.output_file:
+                    skip_load_contents = True
+
+                return fsource(*args, **kwargs,
+                               skip_load_contents=skip_load_contents)
+            except CLIExecError as exc:
+                return CmdOutput(exc.return_value)
+            except SourceNotFound:
+                pass
+
+    def _execute(self, *args, **kwargs):
+        # always try file sources first
+        ret = self.fsource(*args, **kwargs)
+        if ret is not None:
+            return ret
+
+        if HotSOSConfig.data_root != '/':
+            return NullSource()()
+
+        return self.bsource(*args, **kwargs)
 
     def __call__(self, *args, **kwargs):
         """
@@ -490,42 +620,20 @@ class SourceRunner(object):
         A command can have more than one source implementation so we must
         ensure they all have a chance to run.
         """
-        # always try file sources first
-        for fsource in [s for s in self.sources if s.TYPE == "FILE"]:
-            try:
-                return fsource(*args, **kwargs)
-            except CLIExecError as exc:
-                return exc.return_value
-            except SourceNotFound:
-                pass
+        out = self._execute(*args, **kwargs)
+        if self.output_file:
+            if out.source is not None:
+                return out.source
 
-        if HotSOSConfig.data_root != '/':
-            return NullSource()()
+            with open(self.output_file, 'w') as fd:
+                if isinstance(out.value, list):
+                    fd.write(''.join(out.value))
+                else:
+                    fd.write(out.value)
 
-        # binary sources only apply if data_root is system root
-        for bsource in [s for s in self.sources if s.TYPE == "BIN"]:
-            cache = False
-            # NOTE: we currently only support caching commands with no
-            #       args.
-            if not any([args, kwargs]):
-                cache = True
-                out = self.cache.load(self.cmdkey)
-                if out is not None:
-                    return out
+                return self.output_file
 
-            try:
-                out = bsource(*args, **kwargs)
-            except CLIExecError as exc:
-                return exc.return_value
-
-            if cache and out is not None:
-                try:
-                    self.cache.save(self.cmdkey, out)
-                except pickle.PicklingError as exc:
-                    log.info("unable to cache command '%s' output: %s",
-                             self.cmdkey, exc)
-
-            return out
+        return out.value
 
 
 class CLICacheWrapper(object):
@@ -541,7 +649,7 @@ class CLICacheWrapper(object):
         return self.save_f(key, value)
 
 
-class CLIHelper(HostHelpersBase):
+class CLIHelperBase(HostHelpersBase):
 
     def __init__(self):
         self._command_catalog = None
@@ -561,11 +669,11 @@ class CLIHelper(HostHelpersBase):
     def cache_name(self):
         return "commands"
 
-    def cache_load(self, cmdname):
-        return self.cache.get(cmdname)
+    def cache_load(self, key):
+        return self.cache.get(key)
 
-    def cache_save(self, cmdname, output):
-        return self.cache.set(cmdname, output)
+    def cache_save(self, key, output):
+        return self.cache.set(key, output)
 
     @property
     def command_catalog(self):
@@ -712,7 +820,7 @@ class CLIHelper(HostHelpersBase):
                                  last_line_filter='report'),
                  ],
             'ceph_mgr_module_ls':
-                [BinCmd('ceph mgr_module_ls', json_decode=True),
+                [BinCmd('ceph mgr module ls', json_decode=True),
                  # sosreport < 4.2
                  CephJSONFileCmd('sos_commands/ceph/ceph_mgr_module_ls',
                                  json_decode=True),
@@ -728,6 +836,9 @@ class CLIHelper(HostHelpersBase):
             'df':
                 [BinCmd('df'),
                  FileCmd('df')],
+            'dmesg':
+                [BinCmd('dmesg'),
+                 FileCmd('sos_commands/kernel/dmesg')],
             'docker_images':
                 [BinCmd('docker images'),
                  FileCmd('sos_commands/docker/docker_images')],
@@ -736,7 +847,8 @@ class CLIHelper(HostHelpersBase):
                  FileCmd('sos_commands/docker/docker_ps')],
             'dpkg_l':
                 [BinCmd('dpkg -l'),
-                 FileCmd('sos_commands/dpkg/dpkg_-l', safe_decode=True)],
+                 FileCmd('sos_commands/dpkg/dpkg_-l',
+                         decode_error_handling='surrogateecape')],
             'ethtool':
                 [BinCmd('ethtool {interface}'),
                  FileCmd('sos_commands/networking/ethtool_{interface}')],
@@ -764,6 +876,9 @@ class CLIHelper(HostHelpersBase):
             'lscpu':
                 [BinCmd('lscpu'),
                  FileCmd('sos_commands/processor/lscpu')],
+            'lsof_Mnlc':
+                [BinCmd('lsof +M -n -l -c ""'),
+                 FileCmd('sos_commands/process/lsof_M_-n_-l_-c')],
             'lxd_buginfo':
                 [BinCmd('lxd.buginfo'),
                  FileCmd('sos_commands/lxd/lxd.buginfo')],
@@ -773,7 +888,9 @@ class CLIHelper(HostHelpersBase):
             'ns_ip_addr':
                 [BinCmd('ip netns exec {namespace} ip address show'),
                  FileCmd('sos_commands/networking/'
-                         'ip_netns_exec_{namespace}_ip_address_show')],
+                         'ip_netns_exec_{namespace}_ip_address_show'),
+                 FileCmd('sos_commands/networking/namespaces/{namespace}/'
+                         'ip_netns_exec_{namespace}_ip_-d_address_show')],
             'ovn_nbctl_show':
                 [BinCmd('ovn-nbctl show'),
                  FileCmd('sos_commands/ovn_central/ovn-nbctl_show')],
@@ -824,6 +941,14 @@ class CLIHelper(HostHelpersBase):
             'pacemaker_crm_status':
                 [BinCmd('crm status'),
                  FileCmd('sos_commands/pacemaker/crm_status')],
+            'pebble_services':
+                [BinCmd('pebble services'),
+                 # This is how operator charms run it
+                 BinCmd('/charm/bin/pebble services'),
+                 # The following does not exist in sosreport yet but adding
+                 # since it is useful for testing and will hopefully be
+                 # supported in sos at some point.
+                 FileCmd('sos_commands/pebble/pebble_services')],
             'ps':
                 [BinCmd('ps auxwww'),
                  FileCmd('ps')],
@@ -849,7 +974,8 @@ class CLIHelper(HostHelpersBase):
                  FileCmd('sos_commands/kernel/sysctl_-a')],
             'systemctl_status_all':
                 [BinCmd('systemctl status --all'),
-                 FileCmd('sos_commands/systemd/systemctl_status_--all')],
+                 FileCmd('sos_commands/systemd/systemctl_status_--all',
+                         decode_error_handling='backslashreplace')],
             'systemctl_list_units':
                 [BinCmd('systemctl list-units'),
                  FileCmd('sos_commands/systemd/systemctl_list-units')],
@@ -871,12 +997,75 @@ class CLIHelper(HostHelpersBase):
         }
         return self._command_catalog
 
+    @abc.abstractmethod
     def __getattr__(self, cmdname):
-        cmd = self.command_catalog.get(cmdname)
-        if cmd:
-            return SourceRunner(cmdname, cmd, self.cli_cache)
+        """ This is how commands are run. The command is looked up in the
+        catalog and it's runner object is returned. The caller is expetced to
+        call() the returned object to execute the command.
 
-        raise CommandNotFound(cmdname)
+        @param cmdname: name of command we want to execute. This must match a
+        name used to register a handler in the catalog.
+        @return: SourceRunner object.
+        """
+
+
+class CLIHelper(CLIHelperBase):
+    """
+    This is used when we want to have command output as the return value when
+    a command is executed.
+    """
+
+    def __getattr__(self, cmdname):
+        try:
+            return SourceRunner(cmdname, self.command_catalog[cmdname],
+                                self.cli_cache)
+        except KeyError as exc:
+            raise CommandNotFound(cmdname, exc) from exc
+
+
+class CLIHelperFile(CLIHelperBase):
+    """
+    This is used when we want the return value of a command to be a path to a
+    file containing the return value of executing that command.
+
+    This will do one of two things; if the command output originates from a
+    file e.g. a sosreport command output file, it will return the path to that
+    file. If the command is executed as a binary, its output is written to a
+    temporary file and the path to that file is returned.
+    """
+
+    def __init__(self, *args, delete_temp=True, **kwargs):
+        super().__init__(*args, **kwargs)
+        self.delete_temp = delete_temp
+        self._tmp_file_mtime = None
+
+    def __enter__(self):
+        return self
+
+    def __exit__(self, *args, **kwargs):
+        do_delete = (self.delete_temp or
+                     self._tmp_file_mtime ==
+                     os.path.getmtime(self.output_file))
+        if do_delete:
+            os.remove(self.output_file)
+
+        # We want exceptions to be raised
+        return False
+
+    @cached_property
+    def output_file(self):
+        path = tempfile.mktemp(dir=HotSOSConfig.plugin_tmp_dir)
+        pathlib.Path(path).touch()
+        self._tmp_file_mtime = os.path.getmtime(path)
+        return path
+
+    def __getattr__(self, cmdname):
+        try:
+            ret = SourceRunner(cmdname, self.command_catalog[cmdname],
+                               self.cli_cache, output_file=self.output_file)
+            return ret
+        except KeyError as exc:
+            raise CommandNotFound(cmdname, exc) from exc
 
 
 def get_ps_axo_flags_available():
