@@ -1,13 +1,14 @@
+from functools import cached_property
+
 from hotsos.core.host_helpers import (
     APTPackageHelper,
     SnapPackageHelper,
     SystemdHelper,
 )
 from hotsos.core.plugintools import PluginPartBase
-from hotsos.core.utils import cached_property
 
-CORE_APT = ['maas', 'postgres']
-APT_DEPS = ['isc-dhcp', 'bind9']
+CORE_APT = ['maas']
+APT_DEPS = ['isc-dhcp', 'bind9', 'postgres']
 CORE_SNAPS = CORE_APT
 
 SERVICE_EXPRS = [s + '[A-Za-z0-9-]*' for s in CORE_APT + APT_DEPS]
