@@ -1,14 +1,16 @@
-import os
 import glob
+import os
+
+from hotsos.core.config import HotSOSConfig
 from hotsos.core.host_helpers import (
     APTPackageHelper,
+    PebbleHelper,
     SystemdHelper,
 )
 from hotsos.core import (
     host_helpers,
     plugintools,
 )
-from hotsos.core.config import HotSOSConfig
 
 SVC_VALID_SUFFIX = r'[0-9a-zA-Z-_]*'
 MYSQL_SVC_EXPRS = [r'mysql{}'.format(SVC_VALID_SUFFIX)]
@@ -20,6 +22,7 @@ class MySQLChecksBase(plugintools.PluginPartBase):
     def __init__(self, *args, **kwargs):
         super().__init__()
         self.apt_info = APTPackageHelper(core_pkgs=CORE_APT)
+        self.pebble = PebbleHelper(service_exprs=MYSQL_SVC_EXPRS)
         self.systemd = SystemdHelper(service_exprs=MYSQL_SVC_EXPRS)
 
     @property
