@@ -30,17 +30,17 @@ PYTHON_BUILTIN_EXCEPTIONS = [
 #    "IndentationError",
     "IndexError",
     "InterruptedError",
-#    "IOError",
+    "IOError",
     "IsADirectoryError",
 #    "KeyboardInterrupt",
     "KeyError",
     "LookupError",
     "MemoryError",
-#    "ModuleNotFoundError",
+    "ModuleNotFoundError",
     "NameError",
 #    "NotADirectoryError",
     "NotImplementedError",
-#    "OSError",
+    "OSError",
     "OverflowError",
 #    "PendingDeprecationWarning",
     "PermissionError",
@@ -48,7 +48,7 @@ PYTHON_BUILTIN_EXCEPTIONS = [
     "RecursionError",
     "ReferenceError",
     "ResourceWarning",
-#    "RuntimeError",
+    "RuntimeError",
 #    "RuntimeWarning",
 #    "StopAsyncIteration",
 #    "StopIteration",
