@@ -1,8 +1,10 @@
-from datetime import datetime
 import os
+from datetime import datetime
+from functools import cached_property
 
-from hotsos.core import host_helpers
 from hotsos.core.config import HotSOSConfig
+from hotsos.core import host_helpers
+from hotsos.core.log import log
 from hotsos.core.plugins.openstack.exceptions import (
     EXCEPTIONS_COMMON,
     BARBICAN_EXCEPTIONS,
@@ -24,12 +26,12 @@ from hotsos.core.plugins.openstack.exceptions import (
     OVSDBAPP_EXCEPTIONS,
     MASAKARI_EXCEPTIONS,
 )
-from hotsos.core.utils import cached_property
 
 
 # NOTE(tpsilva): when updating this, refer to the Charmed Openstack supported
 # versions page: https://ubuntu.com/openstack/docs/supported-versions
 OST_EOL_INFO = {
+    'antelope': datetime(2026, 4, 30),
     'zed': datetime(2024, 4, 30),
     'yoga': datetime(2027, 4, 30),
     'xena': datetime(2023, 4, 30),
@@ -45,7 +47,20 @@ OST_EOL_INFO = {
 
 
 OST_REL_INFO = {
+    'aodh-common': {
+        'antelope': '1:16.0.0',
+        'zed': '1:15.0.0',
+        'yoga': '1:14.0.0',
+        'xena': '1:13.0.0',
+        'wallaby': '1:12.0.0',
+        'victoria': '11.0.0',
+        'ussuri': '10.0.0',
+        'train': '9.0.0',
+        'stein': '8.0.0',
+        'rocky': '7.0.0',
+        'queens': '6.0.0'},
     'barbican-common': {
+        'antelope': '2:16.0.0',
         'zed': '2:15.0.0',
         'yoga': '2:14.0.0',
         'xena': '1:13.0.0',
@@ -57,6 +72,7 @@ OST_REL_INFO = {
         'rocky': '1:7.0.0',
         'queens': '1:6.0.0'},
     'cinder-common': {
+        'antelope': '2:22.0.0',
         'zed': '2:21.0.0',
         'yoga': '2:20.0.0',
         'xena': '2:19.0.0',
@@ -68,6 +84,7 @@ OST_REL_INFO = {
         'rocky': '2:13.0.0',
         'queens': '2:12.0.0'},
     'designate-common': {
+        'antelope': '1:16.0.0',
         'zed': '1:15.0.0',
         'yoga': '1:14.0.0',
         'xena': '1:13.0.0',
@@ -79,6 +96,7 @@ OST_REL_INFO = {
         'rocky': '1:7.0.0',
         'queens': '1:6.0.0'},
     'glance-common': {
+        'antelope': '2:26.0.0',
         'zed': '2:25.0.0',
         'yoga': '2:24.0.0',
         'xena': '2:23.0.0',
@@ -90,6 +108,7 @@ OST_REL_INFO = {
         'rocky': '2:17.0.0',
         'queens': '2:16.0.0'},
     'heat-common': {
+        'antelope': '1:20.0.0',
         'zed': '1:19.0.0',
         'yoga': '1:18.0.0',
         'xena': '1:17.0.0',
@@ -100,7 +119,12 @@ OST_REL_INFO = {
         'stein': '1:12.0.0',
         'rocky': '1:11.0.0',
         'queens': '1:10.0.0'},
+    'ironic-common': {
+        'antelope': '1:21.2',
+        'zed': '1:20.2.0',
+        'yoga': '1:19.0.0'},
     'keystone': {
+        'antelope': '2:23.0.0',
         'zed': '2:22.0.0',
         'yoga': '2:21.0.0',
         'xena': '2:20.0.0',
@@ -113,26 +137,8 @@ OST_REL_INFO = {
         'queens': '2:13.0.0',
         'pike': '2:12.0.0',
         'ocata': '2:11.0.0'},
-    'nova-common': {
-        'zed': '3:26.0.0',
-        'yoga': '3:25.0.0',
-        'xena': '3:24.0.0',
-        'wallaby': '3:23.0.0',
-        'victoria': '2:22.0.0',
-        'ussuri': '2:21.0.0',
-        'train': '2:20.0.0',
-        'stein': '2:19.0.0',
-        'rocky': '2:18.0.0',
-        'queens': '2:17.0.0',
-        'pike': '2:16.0.0',
-        'ocata': '2:15.0.0',
-        'newton': '2:14.0.0',
-        'mitaka': '2:13.0.0',
-        'liberty': '2:12.0.0',
-        'kilo': '1:2015.1.0',
-        'juno': '1:2014.2.0',
-        'icehouse': '1:2014.1.0'},
     'masakari-common': {
+        'antelope': '15.0.0',
         'zed': '14.0.0',
         'yoga': '13.0.0',
         'xena': '12.0.0',
@@ -143,6 +149,7 @@ OST_REL_INFO = {
         'stein': '7.0.0',
         'rocky': '6.0.0'},
     'neutron-common': {
+        'antelope': '2:22.0.0',
         'zed': '2:21.0.0',
         'yoga': '2:20.0.0',
         'xena': '2:19.0.0',
@@ -161,7 +168,28 @@ OST_REL_INFO = {
         'kilo': '1:2015.1.0',
         'juno': '1:2014.2.0',
         'icehouse': '1:2014.1.0'},
+    'nova-common': {
+        'antelope': '3:27.0.0',
+        'zed': '3:26.0.0',
+        'yoga': '3:25.0.0',
+        'xena': '3:24.0.0',
+        'wallaby': '3:23.0.0',
+        'victoria': '2:22.0.0',
+        'ussuri': '2:21.0.0',
+        'train': '2:20.0.0',
+        'stein': '2:19.0.0',
+        'rocky': '2:18.0.0',
+        'queens': '2:17.0.0',
+        'pike': '2:16.0.0',
+        'ocata': '2:15.0.0',
+        'newton': '2:14.0.0',
+        'mitaka': '2:13.0.0',
+        'liberty': '2:12.0.0',
+        'kilo': '1:2015.1.0',
+        'juno': '1:2014.2.0',
+        'icehouse': '1:2014.1.0'},
     'octavia-common': {
+        'antelope': '1:12.0.0',
         'zed': '1:11.0.0',
         'yoga': '1:10.0.0',
         'xena': '1:9.0.0',
@@ -172,6 +200,7 @@ OST_REL_INFO = {
         'stein': '4.0.0',
         'rocky': '3.0.0'},
     'openstack-dashboard-common': {
+        'antelope': '4:23.1.0',
         'zed': '4:22.2.0',
         'yoga': '4:22.0.0',
         'xena': '4:20.0.0',
@@ -181,8 +210,18 @@ OST_REL_INFO = {
         'train': '3:16.0.0',
         'stein': '3:15.0.0',
         'rocky': '3:14.0.0',
-        'queens': '3:13.0.0'}
-    }
+        'queens': '3:13.0.0'},
+    'placement-common': {
+        'antelope': '1:9.0.0',
+        'zed': '1:8.0.0',
+        'yoga': '1:7.0.0',
+        'xena': '1:6.0.0',
+        'wallaby': '1:5.0.0',
+        'victoria': '4.0.0',
+        'ussuri': '3.0.0',
+        'train': '2.0.0',
+        'stein': '1.0.0'}
+}
 
 OST_EXCEPTIONS = {'barbican': BARBICAN_EXCEPTIONS + CASTELLAN_EXCEPTIONS,
                   'cinder': CINDER_EXCEPTIONS + CASTELLAN_EXCEPTIONS,
@@ -199,11 +238,11 @@ OST_EXCEPTIONS = {'barbican': BARBICAN_EXCEPTIONS + CASTELLAN_EXCEPTIONS,
                   'placement': PLACEMENT_EXCEPTIONS,
                   }
 
-OPENSTACK_LOGS_TS_EXPR = r"^([\d-]+\s+[\d:]+)"
-
 
 class OpenstackConfig(host_helpers.SectionalConfigBase):
-    pass
+
+    def __getattr__(self, key):
+        return self.get(key)
 
 
 class OSTProject(object):
@@ -275,6 +314,11 @@ class OSTProject(object):
     def services(self):
         exprs = self.services_expr
         info = host_helpers.SystemdHelper(service_exprs=exprs)
+        if not info.services:
+            log.debug("no systemd services found for '%s' - trying pebble",
+                      self.name)
+            info = host_helpers.PebbleHelper(service_exprs=exprs)
+
         return info.services
 
     def log_paths(self, include_deprecated_services=True):
@@ -324,36 +368,37 @@ class OSTProjectCatalog(object):
                  systemd_masked_services=['aodh-api'],
                  systemd_extra_services=['apache2'],
                  log_path_overrides={'apache2':
-                                     ['var/log/aodh/aodh-api.log']}),
+                                     ['var/log/aodh/aodh-api.log']})
         self.add('barbican', config={'main': 'barbican.conf'},
                  systemd_masked_services=['barbican-api'],
                  systemd_extra_services=['apache2'],
                  log_path_overrides={'apache2':
-                                     ['var/log/barbican/barbican-api.log']}),
+                                     ['var/log/barbican/barbican-api.log']})
         self.add('ceilometer', config={'main': 'ceilometer.conf'},
-                 systemd_masked_services=['ceilometer-api']),
+                 systemd_masked_services=['ceilometer-api'])
         self.add('cinder', config={'main': 'cinder.conf'},
                  systemd_extra_services=['apache2'],
                  log_path_overrides={'apache2':
-                                     ['var/log/apache2/cinder_*.log']}),
+                                     ['var/log/apache2/cinder_*.log']})
         self.add('designate', config={'main': 'designate.conf'},
-                 systemd_extra_services=['apache2']),
+                 systemd_extra_services=['apache2'])
         self.add('glance', config={'main': 'glance-api.conf'},
-                 systemd_extra_services=['apache2']),
+                 systemd_extra_services=['apache2'])
         self.add('gnocchi', config={'main': 'gnocchi.conf'},
                  systemd_masked_services=['gnocchi-api'],
                  systemd_extra_services=['apache2'],
                  log_path_overrides={'apache2':
-                                     ['var/log/gnocchi/gnocchi-api.log']}),
+                                     ['var/log/gnocchi/gnocchi-api.log']})
         self.add('heat', config={'main': 'heat.conf'},
-                 systemd_extra_services=['apache2']),
+                 systemd_extra_services=['apache2'])
         self.add('horizon', apt_core_alt=['openstack-dashboard'],
-                 systemd_extra_services=['apache2']),
+                 systemd_extra_services=['apache2'])
+        self.add('ironic', config={'main': 'ironic.conf'})
         self.add('keystone', config={'main': 'keystone.conf'},
                  systemd_masked_services=['keystone'],
                  systemd_extra_services=['apache2'],
                  log_path_overrides={'apache2':
-                                     ['var/log/keystone/keystone.log']}),
+                                     ['var/log/keystone/keystone.log']})
         self.add('neutron',
                  config={'main': 'neutron.conf',
                          'openvswitch-agent':
@@ -373,32 +418,32 @@ class OSTProjectCatalog(object):
                  systemd_extra_services=['apache2'],
                  log_path_overrides={'apache2':
                                      ['var/log/apache2/nova-*.log',
-                                      'var/log/nova/nova-api-wsgi.log']}),
+                                      'var/log/nova/nova-api-wsgi.log']})
         self.add('manila', config={'main': 'manila.conf'},
                  systemd_masked_services=['manila-api'],
                  systemd_extra_services=['apache2'],
                  log_path_overrides={'apache2':
-                                     ['var/log/manila/manila-api.log']}),
+                                     ['var/log/manila/manila-api.log']})
         self.add('masakari', config={'main': 'masakari.conf'},
                  systemd_masked_services=['masakari'],
                  systemd_extra_services=['apache2', 'masakari-engine'],
                  log_path_overrides={'apache2':
-                                     ['var/log/apache2/masakari_error.log']}),
+                                     ['var/log/apache2/masakari_error.log']})
         self.add('octavia', config={'main': 'octavia.conf',
                                     'amphora': 'amphora-agent.conf'},
                  systemd_masked_services=['octavia-api'],
                  apt_core_alt=[r'amphora-\S+'],
                  systemd_extra_services=['apache2', 'amphora-agent'],
                  log_path_overrides={'apache2':
-                                     ['var/log/octavia/octavia-api.log']}),
+                                     ['var/log/octavia/octavia-api.log']})
         self.add('placement', config={'main': 'placement.conf'},
                  systemd_masked_services=['placement'],
                  systemd_extra_services=['apache2'],
                  log_path_overrides={'apache2':
-                                     ['var/log/apache2/*error.log']}),
+                                     ['var/log/apache2/*error.log']})
         self.add('swift', config={'main': 'swift-proxy.conf',
                                   'proxy': 'swift-proxy.conf'},
-                 systemd_extra_services=['apache2']),
+                 systemd_extra_services=['apache2'])
 
     def __getitem__(self, name):
         return self._projects[name]
@@ -434,7 +479,7 @@ class OSTProjectCatalog(object):
         for p in self.all.values():
             masked += p.systemd_masked_services
 
-        return masked
+        return sorted(masked)
 
     def add(self, name, *args, **kwargs):
         self._projects[name] = OSTProject(name, *args, **kwargs)
