@@ -1,30 +1,25 @@
 import os
 import re
-import sys
 import subprocess
-
+import sys
 from datetime import datetime
+from functools import cached_property
 
-from hotsos.core.factory import FactoryBase
-from hotsos.core.log import log
-from hotsos.core.utils import (
-    cached_property,
-    mktemp_dump,
-    sorted_dict,
-    seconds_to_date,
-)
 from hotsos.core.config import HotSOSConfig
-from hotsos.core.ycheck.events import YEventCheckerBase
+from hotsos.core.factory import FactoryBase
+from hotsos.core.host_helpers.cli import get_ps_axo_flags_available
 from hotsos.core.host_helpers import (
     APTPackageHelper,
     CLIHelper,
+    CLIHelperFile,
     DPKGVersionCompare,
     HostNetworkingHelper,
+    PebbleHelper,
     SystemdHelper,
     SectionalConfigBase,
-    SVC_EXPR_TEMPLATES,
 )
-from hotsos.core.host_helpers.cli import get_ps_axo_flags_available
+from hotsos.core.log import log
+from hotsos.core.plugins.kernel.net import Lsof
 from hotsos.core.plugins.storage import StorageBase
 from hotsos.core.plugins.storage.bcache import BcacheBase
 from hotsos.core.search import (
@@ -32,8 +27,12 @@ from hotsos.core.search import (
     SequenceSearchDef,
     SearchDef
 )
+from hotsos.core.utils import (
+    sorted_dict,
+    seconds_to_date,
+)
+from hotsos.core.ycheck.events import YEventCheckerBase
 
-CEPH_LOGS_TS_EXPR = r"^([\d-]+)[\sT]([\d:]+)"
 CEPH_SERVICES_EXPRS = [r"ceph-[a-z0-9-]+",
                        r"rados[a-z0-9-:]+"]
 CEPH_PKGS_CORE = [r"ceph",
@@ -42,8 +41,8 @@ CEPH_PKGS_CORE = [r"ceph",
                   ]
 CEPH_PKGS_OTHER = []
 # Add in clients/deps
-for pkg in CEPH_PKGS_CORE:
-    CEPH_PKGS_OTHER.append(r"python3?-{}\S*".format(pkg))
+for ceph_pkg in CEPH_PKGS_CORE:
+    CEPH_PKGS_OTHER.append(r"python3?-{}\S*".format(ceph_pkg))
 
 CEPH_LOGS = "var/log/ceph/"
 
@@ -70,7 +69,7 @@ CEPH_REL_INFO = {
         'luminous': '12.0',
         'kraken': '11.0',
         'jewel': '10.0'},
-    }
+}
 
 CEPH_POOL_TYPE = {1: 'replicated', 3: 'erasure-coded'}
 
@@ -82,7 +81,7 @@ def csv_to_set(f):
     def csv_to_set_inner(*args, **kwargs):
         val = f(*args, **kwargs)
         if val is not None:
-            return set([v.strip(',') for v in val.split()])
+            return set(v.strip(',') for v in val.split())
 
         return set([])
 
@@ -132,9 +131,6 @@ class CephConfig(SectionalConfigBase):
 
 class CephCrushMap(object):
 
-    def __init__(self):
-        self.cli = CLIHelper()
-
     def _filter_pools_by_rule(self, pools, crush_rule):
         res_pool = []
         for pool in pools:
@@ -146,11 +142,11 @@ class CephCrushMap(object):
 
     @cached_property
     def osd_crush_dump(self):
-        return self.cli.ceph_osd_crush_dump_json_decoded() or {}
+        return CLIHelper().ceph_osd_crush_dump_json_decoded() or {}
 
     @cached_property
     def ceph_report(self):
-        return self.cli.ceph_report_json_decoded() or {}
+        return CLIHelper().ceph_report_json_decoded() or {}
 
     @cached_property
     def rules(self):
@@ -316,6 +312,21 @@ class CephCrushMap(object):
         pools = self.ceph_report['osdmap']['pools']
         return [p for p in pools if p.get('pg_autoscale_mode') != 'on']
 
+    @cached_property
+    def is_rgw_using_civetweb(self):
+        if not self.ceph_report:
+            return []
+
+        try:
+            rgws = self.ceph_report['servicemap']['services']['rgw']['daemons']
+            for _, outer_d in rgws.items():
+                if isinstance(outer_d, dict):
+                    if outer_d['metadata']['frontend_type#0'] == 'civetweb':
+                        return True
+        except (ValueError, KeyError):
+            pass
+        return False
+
 
 class CephCluster(object):
     OSD_META_LIMIT_PERCENT = 5
@@ -327,29 +338,28 @@ class CephCluster(object):
 
     def __init__(self):
         self.crush_map = CephCrushMap()
-        self.cli = CLIHelper()
 
     @cached_property
     def health_status(self):
-        status = self.cli.ceph_status_json_decoded()
+        status = CLIHelper().ceph_status_json_decoded()
         if status:
             return status['health']['status']
 
     @cached_property
     def mon_dump(self):
-        return self.cli.ceph_mon_dump_json_decoded() or {}
+        return CLIHelper().ceph_mon_dump_json_decoded() or {}
 
     @cached_property
     def osd_dump(self):
-        return self.cli.ceph_osd_dump_json_decoded() or {}
+        return CLIHelper().ceph_osd_dump_json_decoded() or {}
 
     @cached_property
     def pg_dump(self):
-        return self.cli.ceph_pg_dump_json_decoded() or {}
+        return CLIHelper().ceph_pg_dump_json_decoded() or {}
 
     @cached_property
     def ceph_mgr_module_ls(self):
-        return self.cli.ceph_mgr_module_ls() or {}
+        return CLIHelper().ceph_mgr_module_ls() or {}
 
     @cached_property
     def mons(self):
@@ -377,11 +387,11 @@ class CephCluster(object):
 
     @cached_property
     def osd_df_tree(self):
-        return self.cli.ceph_osd_df_tree_json_decoded() or {}
+        return CLIHelper().ceph_osd_df_tree_json_decoded() or {}
 
     @cached_property
     def ceph_df(self):
-        return self.cli.ceph_df_json_decoded() or {}
+        return CLIHelper().ceph_df_json_decoded() or {}
 
     @cached_property
     def osds(self):
@@ -411,11 +421,6 @@ class CephCluster(object):
         the resulting dict is keyed by daemon type otherwise it is keyed by
         version (and only versions for that daemon type.)
         """
-        out = CLIHelper().ceph_versions()
-        if not out:
-            return
-
-        out_path = mktemp_dump('\n'.join(out))
         versions = {}
         s = FileSearcher()
         body = SearchDef(r"\s+\"ceph version (\S+) .+ (\S+) "
@@ -430,19 +435,20 @@ class CephCluster(object):
                                    end=SearchDef(r"^\s+\"\S+\":\s+{"),
                                    tag='versions')
 
-        s.add(sd, path=out_path)
-        for section in s.run().find_sequence_sections(sd).values():
-            _versions = {}
-            for result in section:
-                if result.tag == sd.start_tag:
-                    _daemon_type = result.get(1)
-                    versions[_daemon_type] = _versions
-                elif result.tag == sd.body_tag:
-                    version = result.get(1)
-                    rname = result.get(2)
-                    amount = result.get(3)
-                    _versions[version] = {'release_name': rname,
-                                          'count': int(amount)}
+        with CLIHelperFile() as cli:
+            s.add(sd, path=cli.ceph_versions())
+            for section in s.run().find_sequence_sections(sd).values():
+                _versions = {}
+                for result in section:
+                    if result.tag == sd.start_tag:
+                        _daemon_type = result.get(1)
+                        versions[_daemon_type] = _versions
+                    elif result.tag == sd.body_tag:
+                        version = result.get(1)
+                        rname = result.get(2)
+                        amount = result.get(3)
+                        _versions[version] = {'release_name': rname,
+                                              'count': int(amount)}
 
         # If specific daemon_type provided only return version for that type
         # otherwise all.
@@ -540,13 +546,13 @@ class CephCluster(object):
 
         return laggy_pgs
 
-    def pool_id_to_name(self, id):
+    def pool_id_to_name(self, pool_id):
         if not self.osd_dump:
             return
 
         pools = self.osd_dump.get('pools', [])
         for pool in pools:
-            if pool['pool'] == int(id):
+            if pool['pool'] == int(pool_id):
                 return pool['pool_name']
 
     @cached_property
@@ -560,10 +566,10 @@ class CephCluster(object):
                 pg_id = pg['pgid']
                 pg_pool_id = pg_id.partition('.')[0]
                 _large_omap_pgs[pg['pgid']] = {
-                        'pool': self.pool_id_to_name(pg_pool_id),
-                        'last_scrub_stamp': pg['last_scrub_stamp'],
-                        'last_deep_scrub_stamp': pg['last_deep_scrub_stamp']
-                        }
+                    'pool': self.pool_id_to_name(pg_pool_id),
+                    'last_scrub_stamp': pg['last_scrub_stamp'],
+                    'last_deep_scrub_stamp': pg['last_deep_scrub_stamp']
+                }
 
         return _large_omap_pgs
 
@@ -641,7 +647,8 @@ class CephCluster(object):
                 if daemon_type in exclude_daemons:
                     continue
 
-            unique_versions[daemon_type] = list(set(versions[daemon_type]))
+            unique_versions[daemon_type] = sorted(
+                list(set(versions[daemon_type])))
 
         return unique_versions
 
@@ -684,7 +691,7 @@ class CephCluster(object):
 
     @cached_property
     def osdmaps_count(self):
-        report = self.cli.ceph_report_json_decoded()
+        report = CLIHelper().ceph_report_json_decoded()
         if not report:
             return 0
 
@@ -726,13 +733,27 @@ class CephCluster(object):
 
         return sorted_dict(_osds_pgs, key=lambda e: e[1], reverse=True)
 
+    @cached_property
+    def ssds_using_bcache(self):
+        report = CLIHelper().ceph_report_json_decoded()
+        if not report:
+            return []
+
+        ssd_osds_using_bcache = []
+        for osd in report['osd_metadata']:
+            if osd['bluestore_bdev_type'] == 'ssd' and \
+                    osd['bluestore_bdev_rotational'] == '0' and \
+                    re.search("bcache", osd['bluestore_bdev_devices']):
+                ssd_osds_using_bcache.append(osd['id'])
+
+        return sorted(ssd_osds_using_bcache)
+
 
 class CephDaemonBase(object):
 
     def __init__(self, daemon_type):
         self.daemon_type = daemon_type
         self.id = None
-        self.cli = CLIHelper()
         self.date_in_secs = self.get_date_secs()
 
     @classmethod
@@ -756,20 +777,21 @@ class CephDaemonBase(object):
         s = FileSearcher()
         # columns: USER PID %CPU %MEM VSZ RSS TTY STAT START TIME COMMAND
         if self.id is not None:
-            id = r"--id\s+{}".format(self.id)
+            ceph_id = r"--id\s+{}".format(self.id)
         else:
-            id = ''
+            ceph_id = ''
 
         expr = (r"\S+\s+\d+\s+\S+\s+\S+\s+\d+\s+(\d+)\s+.+/ceph-{}\s+.+{}\s+.+"
-                r".+".format(self.daemon_type, id))
+                r".+".format(self.daemon_type, ceph_id))
         sd = SearchDef(expr)
-        ps_out = mktemp_dump('\n'.join(self.cli.ps()))
-        s.add(sd, path=ps_out)
-        rss = 0
-        # we only expect one result
-        for result in s.run().find_by_path(ps_out):
-            rss = int(int(result.get(1)) / 1024)
-            break
+        with CLIHelperFile() as cli:
+            ps_out = cli.ps()
+            s.add(sd, path=ps_out)
+            rss = 0
+            # we only expect one result
+            for result in s.run().find_by_path(ps_out):
+                rss = int(int(result.get(1)) / 1024)
+                break
 
         return "{}M".format(rss)
 
@@ -788,12 +810,12 @@ class CephDaemonBase(object):
 
         ps_info = []
         daemon = "ceph-{}".format(self.daemon_type)
-        for line in self.cli.ps_axo_flags():
+        for line in CLIHelper().ps_axo_flags():
             ret = re.compile(daemon).search(line)
             if not ret:
                 continue
 
-            expt_tmplt = SVC_EXPR_TEMPLATES["absolute"]
+            expt_tmplt = SystemdHelper.PS_CMD_EXPR_TEMPLATES['absolute']
             ret = re.compile(expt_tmplt.format(daemon)).search(line)
             if ret:
                 ps_info.append(ret.group(0))
@@ -808,7 +830,7 @@ class CephDaemonBase(object):
                 osd_start = ' '.join(cmd.split()[13:17])
                 if self.date_in_secs and osd_start:
                     osd_start_secs = self.get_date_secs(datestring=osd_start)
-                    osd_uptime_secs = (self.date_in_secs - osd_start_secs)
+                    osd_uptime_secs = self.date_in_secs - osd_start_secs
                     osd_uptime_str = seconds_to_date(osd_uptime_secs)
                     _etime = osd_uptime_str
 
@@ -836,9 +858,9 @@ class CephRGW(CephDaemonBase):
 
 class CephOSD(CephDaemonBase):
 
-    def __init__(self, id, fsid=None, device=None, dump=None):
+    def __init__(self, ceph_id, fsid=None, device=None, dump=None):
         super().__init__('osd')
-        self.id = id
+        self.id = ceph_id
         self.fsid = fsid
         self.device = device
         self.dump = dump
@@ -861,7 +883,7 @@ class CephOSD(CephDaemonBase):
 
     @cached_property
     def devtype(self):
-        osd_tree = self.cli.ceph_osd_df_tree_json_decoded()
+        osd_tree = CLIHelper().ceph_osd_df_tree_json_decoded()
         if not osd_tree:
             return
 
@@ -881,9 +903,10 @@ class CephChecksBase(StorageBase):
         self.bcache = BcacheBase()
         self.apt = APTPackageHelper(core_pkgs=CEPH_PKGS_CORE,
                                     other_pkgs=CEPH_PKGS_OTHER)
+        self.pebble = PebbleHelper(service_exprs=CEPH_SERVICES_EXPRS)
         self.systemd = SystemdHelper(service_exprs=CEPH_SERVICES_EXPRS)
         self.cluster = CephCluster()
-        self.cli = CLIHelper()
+        self.lsof = Lsof()
 
     @property
     def summary_subkey(self):
@@ -915,19 +938,19 @@ class CephChecksBase(StorageBase):
     def days_to_eol(self):
         if self.release_name != 'unknown':
             eol = CEPH_EOL_INFO[self.release_name]
-            today = datetime.fromtimestamp(int(CLIHelper().date()))
+            today = datetime.utcfromtimestamp(int(CLIHelper().date()))
             delta = (eol - today).days
             return delta
 
-    def _get_bind_interfaces(self, type):
+    def _get_bind_interfaces(self, iface_type):
         """
         For the given config network type determine what interface ceph is
         binding to.
 
-        @param type: cluster or public
+        @param iface_type: cluster or public
         """
-        net = self.ceph_config.get('{} network'.format(type))
-        addr = self.ceph_config.get('{} addr'.format(type))
+        net = self.ceph_config.get('{} network'.format(iface_type))
+        addr = self.ceph_config.get('{} addr'.format(iface_type))
         if not any([net, addr]):
             return {}
 
@@ -939,7 +962,7 @@ class CephChecksBase(StorageBase):
             port = nethelp.get_interface_with_addr(addr)
 
         if port:
-            return {type: port}
+            return {iface_type: port}
 
     @cached_property
     def _ceph_bind_interfaces(self):
@@ -948,8 +971,8 @@ class CephChecksBase(StorageBase):
         The dict has the form {<type>: [<port>, ...]}
         """
         interfaces = {}
-        for type in ['cluster', 'public']:
-            ret = self._get_bind_interfaces(type)
+        for iface_type in ['cluster', 'public']:
+            ret = self._get_bind_interfaces(iface_type)
             if ret:
                 interfaces.update(ret)
 
@@ -974,31 +997,29 @@ class CephChecksBase(StorageBase):
         Returns a list of CephOSD objects for osds found on the local host.
         """
         osds = []
-        out = self.cli.ceph_volume_lvm_list()
-        if not out:
-            return osds
 
-        out_path = mktemp_dump('\n'.join(out))
         s = FileSearcher()
         sd = SequenceSearchDef(start=SearchDef(r"^=+\s+osd\.(\d+)\s+=+.*"),
                                body=SearchDef([r"\s+osd\s+(fsid)\s+(\S+)\s*",
                                                r"\s+(devices)\s+([\S]+)\s*"]),
                                tag="ceph-lvm")
-        s.add(sd, path=out_path)
-        for results in s.run().find_sequence_sections(sd).values():
-            id = None
-            fsid = None
-            dev = None
-            for result in results:
-                if result.tag == sd.start_tag:
-                    id = int(result.get(1))
-                elif result.tag == sd.body_tag:
-                    if result.get(1) == "fsid":
-                        fsid = result.get(2)
-                    elif result.get(1) == "devices":
-                        dev = result.get(2)
+        with CLIHelperFile() as cli:
+            fout = cli.ceph_volume_lvm_list()
+            s.add(sd, path=fout)
+            for results in s.run().find_sequence_sections(sd).values():
+                osdid = None
+                fsid = None
+                dev = None
+                for result in results:
+                    if result.tag == sd.start_tag:
+                        osdid = int(result.get(1))
+                    elif result.tag == sd.body_tag:
+                        if result.get(1) == "fsid":
+                            fsid = result.get(2)
+                        elif result.get(1) == "devices":
+                            dev = result.get(2)
 
-            osds.append(CephOSD(id, fsid, dev))
+                osds.append(CephOSD(osdid, fsid, dev))
 
         return osds
 
@@ -1060,6 +1081,23 @@ class CephChecksBase(StorageBase):
 
         return False
 
+    @cached_property
+    def linked_with_tcmalloc(self):
+        """
+        Checks that each ceph-osd process has libtcmalloc linked.
+
+        Returns True if every OSD has it linked, otherwise False.
+        """
+        osds = {}
+        tcmalloc_osds = 0
+        for row in self.lsof:
+            if row.COMMAND == 'ceph-osd':
+                osds[row.PID] = 1
+                if re.search("libtcmalloc", row.NAME):
+                    tcmalloc_osds += 1
+
+        return len(osds) == tcmalloc_osds
+
 
 class CephDaemonCommand(object):
     """
@@ -1147,4 +1185,4 @@ class CephEventChecksBase(CephChecksBase, YEventCheckerBase):
     @property
     def summary(self):
         # mainline all results into summary root
-        return self.run_checks()
+        return self.load_and_run()
