@@ -1,10 +1,13 @@
-import re
 import os
+import re
+from functools import cached_property
 
-from hotsos.core.log import log
 from hotsos.core.config import HotSOSConfig
-from hotsos.core.host_helpers import CLIHelper, SYSCtlFactory
-from hotsos.core.utils import cached_property, mktemp_dump
+from hotsos.core.host_helpers import (
+    CLIHelper, CLIHelperFile,
+    SYSCtlFactory, UptimeHelper
+)
+from hotsos.core.log import log
 from hotsos.core.search import (
     FileSearcher, SearchDef,
     SequenceSearchDef
@@ -81,14 +84,19 @@ class SystemBase(object):
     def hostname(self):
         return CLIHelper().hostname()
 
+    @cached_property
+    def uptime(self):
+        return str(UptimeHelper())
+
     @cached_property
     def os_release_name(self):
         data_source = os.path.join(HotSOSConfig.data_root, "etc/lsb-release")
         if os.path.exists(data_source):
-            for line in open(data_source).read().split():
-                ret = re.compile(r"^DISTRIB_CODENAME=(.+)").match(line)
-                if ret:
-                    return "ubuntu {}".format(ret[1])
+            with open(data_source) as fd:
+                for line in fd.read().split():
+                    ret = re.compile(r"^DISTRIB_CODENAME=(.+)").match(line)
+                    if ret:
+                        return "ubuntu {}".format(ret[1])
 
     @cached_property
     def virtualisation_type(self):
@@ -128,8 +136,7 @@ class SystemBase(object):
             if ret:
                 if int(ret[1]) == 0:
                     return False
-                else:
-                    return True
+                return True
 
         return False
 
@@ -175,42 +182,44 @@ class SystemBase(object):
             r".*not attached to.*(Ubuntu (Pro|Advantage)|UA).*",
             tag="not-attached")
 
-        f = mktemp_dump("".join(CLIHelper().pro_status()))
-        s.add(not_attached_def, f)
-        s.add(service_status_seqdef, f)
-        s.add(account_status_seqdef, f)
-        results = s.run()
+        with CLIHelperFile() as cli:
+            f = cli.pro_status()
+            s.add(not_attached_def, f)
+            s.add(service_status_seqdef, f)
+            s.add(account_status_seqdef, f)
+            results = s.run()
+            if results.find_by_tag("not-attached"):
+                return {"status": "not-attached"}
 
-        if results.find_by_tag("not-attached"):
-            return {"status": "not-attached"}
+            ssects = results.find_sequence_sections(service_status_seqdef)
+            asects = results.find_sequence_sections(account_status_seqdef)
+            if not all([ssects, asects]):
+                log.debug("badness: `pro status` does not match "
+                          "the expected format")
+                return {"status": "error"}
 
-        ssects = results.find_sequence_sections(service_status_seqdef)
-        asects = results.find_sequence_sections(account_status_seqdef)
-        if not all([ssects, asects]):
-            log.debug("badness: `pro status` does not match "
-                      "the expected format")
-            return {"status": "error"}
+            result = {}
+            result["status"] = "attached"
+            result["services"] = {}
 
-        result = {}
-        result["status"] = "attached"
-        result["services"] = {}
+            for sid in ssects:
+                result["services"] = {**result["services"], **{
+                    v.get(1): {
+                        "entitled": v.get(2),
+                        "status": v.get(3)
+                    }
+                    for v in ssects[sid]
+                    if v.tag == service_status_seqdef.body_tag
+                }}
 
-        for id in ssects:
-            result["services"] = {**result["services"], **{
-                v.get(1): {
-                    "entitled": v.get(2),
-                    "status": v.get(3)
-                }
-                for v in ssects[id] if v.tag == service_status_seqdef.body_tag
-            }}
+            for sid in asects:
+                result = {**result, **{
+                    re.sub(r'\W+', '_',
+                           v.get(1).strip()).lower(): v.get(2).strip()
+                    for v in asects[sid]
+                }}
 
-        for id in asects:
-            result = {**result, **{
-                re.sub(r'\W+', '_', v.get(1).strip()).lower(): v.get(2).strip()
-                for v in asects[id]
-            }}
-
-        return result
+            return result
 
     @cached_property
     def sysctl_all(self):
