@@ -1,6 +1,11 @@
+from functools import cached_property
+
 from hotsos.core.plugintools import PluginPartBase
-from hotsos.core.host_helpers import SystemdHelper, SnapPackageHelper
-from hotsos.core.utils import cached_property
+from hotsos.core.host_helpers import (
+    PebbleHelper,
+    SnapPackageHelper,
+    SystemdHelper,
+)
 
 CORE_SNAPS = ['vault']
 SERVICE_EXPRS = [s + '[A-Za-z0-9-]*' for s in CORE_SNAPS]
@@ -12,6 +17,7 @@ class VaultChecksBase(PluginPartBase):
         super().__init__(*args, **kwargs)
         self.snaps = SnapPackageHelper(core_snaps=CORE_SNAPS)
         self.systemd = SystemdHelper(service_exprs=SERVICE_EXPRS)
+        self.pebble = PebbleHelper(service_exprs=SERVICE_EXPRS)
 
     @cached_property
     def vault_installed(self):
