@@ -1,14 +1,15 @@
-from .common import YDefsSection  # noqa: F403,F401
 # the following are done to allow the decorators to do their thing
 from hotsos.core.ycheck.engine.properties import (  # noqa: F403,F401
     checks,
     conclusions,
-    input,
+    inputdef,
     search,
-    vars,
+    vardef,
 )
 import hotsos.core.ycheck.engine.properties.requires.requires  # noqa:F403,F401
 from hotsos.core.ycheck.engine.properties.requires.types import (  \
     # noqa:F403,F401
     config,
 )
+
+from .common import YDefsSection  # noqa: F403,F401
