@@ -1,14 +1,13 @@
 import abc
 import builtins
-import inspect
 import importlib
+import inspect
 
 from propertree import (
     PTreeOverrideBase,
     PTreeMappedOverrideBase,
     PTreeSection,
 )
-
 from hotsos.core.log import log
 
 YPropertiesCatalog = []
@@ -78,7 +77,10 @@ class YDefsContext(object):
 
     def __setattr__(self, key, value):
         if key != '_ydefs_context':
-            log.debug("%s setting %s=%s", self.__class__.__name__, key, value)
+            # don't print values as they can be very large and e.g. search
+            # results will be expanded re-duplicated.
+            log.debug("%s setting %s with value of type '%s'",
+                      self.__class__.__name__, key, type(value))
             self._ydefs_context[key] = value
             return
 
@@ -92,7 +94,7 @@ class PropertyCacheRefResolver(object):
     """
     This class is used to resolve string references to property cache entries.
     """
-    def __init__(self, refstr, vars=None, checks=None):
+    def __init__(self, refstr, pcrr_vars=None, checks=None):
         log.debug("%s: resolving '%s'", self.__class__.__name__, refstr)
         self.refstr = refstr
         if not self.is_valid_cache_ref(refstr):
@@ -100,7 +102,7 @@ class PropertyCacheRefResolver(object):
                    format(refstr))
             raise Exception(msg)
 
-        self.vars = vars
+        self.vars = pcrr_vars
         self.checks = checks
         if self.reftype == 'checks' and checks is None:
             msg = ("{} is a checks cache reference but checks dict not "
@@ -119,11 +121,11 @@ class PropertyCacheRefResolver(object):
         """
         if self.refstr.startswith('$'):
             return "variable"
-        elif self.refstr.startswith('@checks.'):
+        if self.refstr.startswith('@checks.'):
             # This is an implementation of YPropertyChecks
             return "checks"
-        else:
-            raise Exception("unknown ref type")
+
+        raise Exception("unknown ref type")
 
     @property
     def _ref_body(self):
@@ -189,27 +191,67 @@ class PropertyCacheRefResolver(object):
 
         return _key.partition(':')[2]
 
-    def apply_renderer_function(self, value):
+    @staticmethod
+    def apply_renderer(value, func):
         """
-        The last section of a ref string can be a colon followed by a function
-        name which itself can be one of two things; any method supported by
-        builtins or "comma_join".
+        A cache reference or import path can be suffixed with :<function>
+        where <function> can be one of two things; either a builtins function
+        or one of the extras defined here. Obviously not all builtins can be
+        used and their use depends on the type() of value.
+
+        @param value: value to apply renderer function to
+        @param func: function to apply to value
         """
-        func = self.property_cache_value_renderer_function
-        if func:
-            if func == "comma_join":
-                # needless to say this will only work with lists, dicts etc.
-                return ', '.join(value)
+        extras = {'comma_join': {
+                    'requirements': [lambda value: isinstance(value, list),
+                                     lambda value: isinstance(value, dict)],
+                    'action': lambda value: ', '.join(value)},
+                  'unique_comma_join': {
+                    'requirements': [lambda value: isinstance(value, list),
+                                     lambda value: isinstance(value, dict)],
+                    'action': lambda value: ', '.join(sorted(set(value)))},
+                  'first': {
+                    'requirements': [lambda value: isinstance(value, list)],
+                    'action': lambda value: value[0]}}
 
-            return getattr(builtins, func)(value)
+        if func in extras:
+            if not any(req(value) for req in extras[func]['requirements']):
+                log.warning("attempted to apply '%s' to value of "
+                            "type %s", func, type(value))
+                return value
 
-        return value
+            return extras[func]['action'](value)
+
+        return getattr(builtins, func)(value)
+
+    def _get_search_result_group(self, cache_key):
+        """
+        Extract values at the given group index from all search results.
+        """
+        val = []
+        group = int(cache_key.partition('results_group_')[2])
+        for result in self.checks[self.check_name]._search_results:
+            if len(result) >= group:
+                val.append(result.get(group))
+
+        return sorted(val)
 
     def resolve(self):
+        """
+        Resolve value associated with a check property result or variable.
+
+        Values are then run through an optional renderer function.
+        """
         if self.reftype == 'checks':
-            check_cache = self.checks[self.check_name].cache
-            property_cache = getattr(check_cache, self.property_name)
-            val = getattr(property_cache, self.property_cache_key)
+            cache_key = self.property_cache_key
+            # This provides an interface to extract the values of specific
+            # search result groups.
+            if cache_key.startswith('results_group_'):
+                val = self._get_search_result_group(cache_key)
+            else:
+                check_cache = self.checks[self.check_name].cache
+                property_cache = getattr(check_cache, self.property_name)
+                val = getattr(property_cache, cache_key)
         else:
             varname = self.refstr.partition("$")[2]
             varname = varname.partition(':')[0]
@@ -218,7 +260,12 @@ class PropertyCacheRefResolver(object):
         if val is None:
             return
 
-        return self.apply_renderer_function(val)
+        func = self.property_cache_value_renderer_function
+        if not func:
+            # noop
+            return val
+
+        return self.apply_renderer(val, func)
 
 
 class PropertyCache(object):
@@ -556,8 +603,8 @@ class LogicalCollectionHandler(abc.ABC):
         result.
         """
 
-    def _is_op_group(self, property):
-        return property._override_name in self.VALID_GROUP_KEYS
+    def _is_op_group(self, prop):
+        return prop._override_name in self.VALID_GROUP_KEYS
 
     def eval_op_group_items(self, logical_op_group):
         """
