@@ -1,12 +1,15 @@
-from hotsos.core.plugintools import summary_entry_offset as idx
 from hotsos.core.plugins.kubernetes import KubernetesChecksBase
+from hotsos.core.plugintools import summary_entry_offset as idx
 
 
 class KubernetesSummary(KubernetesChecksBase):
 
     @idx(0)
     def __summary_services(self):
-        return self.systemd.summary
+        if self.systemd.services:
+            return self.systemd.summary
+        if self.pebble.services:
+            return self.pebble.summary
 
     @idx(1)
     def __summary_snaps(self):
