@@ -1,5 +1,5 @@
-from hotsos.core.plugintools import summary_entry_offset as idx
 from hotsos.core.plugins.mysql import MySQLChecksBase
+from hotsos.core.plugintools import summary_entry_offset as idx
 
 
 class MySQLSummary(MySQLChecksBase):
@@ -8,6 +8,8 @@ class MySQLSummary(MySQLChecksBase):
     def __summary_services(self):
         if self.systemd.services:
             return self.systemd.summary
+        if self.pebble.services:
+            return self.pebble.summary
 
     @idx(1)
     def __summary_dpkg(self):
