@@ -1,6 +1,6 @@
-from hotsos.core.plugintools import summary_entry_offset as idx
 from hotsos.core.plugins.openstack.common import OpenstackChecksBase
 from hotsos.core.plugins.openstack.neutron import NeutronHAInfo
+from hotsos.core.plugintools import summary_entry_offset as idx
 
 
 class OpenstackSummary(OpenstackChecksBase):
@@ -15,6 +15,8 @@ class OpenstackSummary(OpenstackChecksBase):
         """Get string info for running services."""
         if self.systemd.services:
             return self.systemd.summary
+        if self.pebble.services:
+            return self.pebble.summary
 
     @idx(2)
     def __summary_dpkg(self):
@@ -37,9 +39,9 @@ class OpenstackSummary(OpenstackChecksBase):
         for router in ha_info.ha_routers:
             state = router.ha_state
             if state in routers:
-                routers[state].append(router.uuid)
+                routers[state] += 1
             else:
-                routers[state] = [router.uuid]
+                routers[state] = 1
 
         if routers:
             return routers
