@@ -44,8 +44,7 @@ class SystemSummary(SystemChecksBase):
     def __summary_unattended_upgrades(self):
         if self.unattended_upgrades_enabled:
             return "ENABLED"
-        else:
-            return "disabled"
+        return "disabled"
 
     @idx(7)
     def __summary_date(self):
@@ -54,3 +53,7 @@ class SystemSummary(SystemChecksBase):
     @idx(8)
     def __summary_ubuntu_pro(self):
         return self.ubuntu_pro_status
+
+    @idx(9)
+    def __summary_uptime(self):
+        return self.uptime
