@@ -1,5 +1,5 @@
-from hotsos.core.plugintools import summary_entry_offset as idx
 from hotsos.core.plugins.vault import VaultChecksBase
+from hotsos.core.plugintools import summary_entry_offset as idx
 
 
 class VaultSummary(VaultChecksBase):
@@ -8,6 +8,8 @@ class VaultSummary(VaultChecksBase):
     def __summary_services(self):
         if self.systemd.services:
             return self.systemd.summary
+        if self.pebble.services:
+            return self.pebble.summary
 
     @idx(1)
     def __summary_snaps(self):
